#include <stdlib.h>
#include <stdio.h>
#include "oware.h"


void printboard(int thisside[], int otherside[])
{
  printf("  ");
  for (int i=houses-1; i>=0; i--)
    printf("%3d",otherside[i]);
  printf("\n%3d%*c%3d\n  ",otherside[houses],houses*3-2,' ',thisside[houses]);
  for (int i=0; i<houses; i++)
    printf("%3d",thisside[i]);
}

/* 1 if side is empty, 0 otherwise */
int empty(int side[])
{
  for (int i=0; i<houses; i++)
    if (side[i])
      return 0;
  return 1;
}

/* perform move (0-5) on position thisside/otherside, but do not check
   some boundary conditions */
int ply(int move, int thisside[], int otherside[])
{
  int seeds, sow;
  int thissidebackup[houses+1];
  int othersidebackup[houses+1];

  for (int i=0; i<=houses; i++) { /* if the move is invalid, we need to restore */
    thissidebackup[i]=thisside[i];
    othersidebackup[i]=otherside[i];
  }
  if (move<0||move>=houses) {
    /* fprintf(stderr,"move %d out of range\n",move);*/
    return 0;
  }
  seeds=thisside[move];
  thisside[move]=0;
  if (seeds==0) {
    /* fprintf(stderr,"no seeds at %d\n",move);*/
    return 0;
  }
  sow=move+1;
  while (seeds>0) {
    /* sow in your own houses */
    for (; seeds>0 && sow<houses; sow++)
      if (sow!=move) {
        thisside[sow]++;
        seeds--;
      }
    /* sow in oponent houses */
    for (sow=0; seeds>0 && sow<houses; sow++) {
        otherside[sow]++;
        seeds--;
    }
    if (seeds==0) {
       /* make backup, in case of grand slam */
      int tshbackup=thisside[houses];
      int othersidebackup1[houses];
      for (int i=0; i<houses; i++)
        othersidebackup1[i]=otherside[i];
      /* now check if we capture */
      for (sow--; sow>=0 && (otherside[sow]==2 || otherside[sow]==3); sow--) {
        thisside[houses]+=otherside[sow];
        otherside[sow]=0;
      }
      /* grand slam rule: legal but no capture results */
      if (empty(otherside)) {
        thisside[houses]=tshbackup;
        for (int i=0; i<houses; i++)
          otherside[i]=othersidebackup1[i];
      }
    }
    sow=0;
  }
  /* must make a move such that the opponent has seeds */
  if (empty(otherside)) {
    for (int i=0; i<=houses; i++) {
      thisside[i]=thissidebackup[i];
      otherside[i]=othersidebackup[i];
    }
    /* fprintf(stderr,"other side empty\n");*/
    return 0;
  }
  return 1;
}
  
int canmove(int thisside[], int otherside[])
{
  int ts[houses+1];
  int os[houses+1];
  for (int i=0; i<=houses; i++) {
    ts[i]=thisside[i];
    os[i]=otherside[i];
  }
  for (int i=0; i<houses; i++)
    if (ply(i, ts, os))
      return 1;
  return 0;
}

/* perform move (0-5) on position thisside/otherside, including
   resolving game-ending conditions */
int plyfull(int move, int thisside[], int otherside[])
{
  if (ply(move,thisside,otherside)==0) {
    return 0;
  }
  if (!canmove(otherside,thisside)) {
    for (int i=0; i<houses; i++) {
      thisside[houses] += thisside[i];
      thisside[i]=0;
      otherside[houses] += otherside[i];
      otherside[i]=0;
    }
  }
  return 1;
}

/* initialize thisside and otherside from the command line arguments */
int cmdline2pos(int argc, char *argv[], int thisside[], int otherside[])
{
  if (argc != 2*houses+2)
    return 0;
  for (int i=0; i<argc; i++)
    if (i<=houses)
      thisside[i]=atoi(argv[i]);
    else
      otherside[i-houses-1]=atoi(argv[i]);
  return 1;
}
